#include <Adafruit_NeoPixel.h> //NeoPixel LEDを光らせるためのライブラリ
#include <math.h> //数学の関数を使うためのライブラリ
#define LED_PIN 2 //LEDのDINを接続したArduinoのピン
#define NUM_LEDS 10 //LEDの数
#define MIC_PIN A0 //センサーを接続したArduinoのピン
#define SAMPLE_WINDOW 10
#define PEAK_HANG 24
#define PEAK_FALL 4
#define INPUT_FLOOR 10
#define INPUT_CEILING 300
byte peak = 16;
unsigned int sample;

byte dotCount = 0;
byte dotHangCount = 0;

//NeoPixelライブラリの初期化
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);
void setup(){ //Arduino初期設定：電源ON時・リセット時に実行される
  strip.begin();
  strip.show(); //LEDを「オフ」に設定
}

void loop(){ //実行したいプログラムを記述：繰返し実行される
  unsigned long startMillis = millis();
  float peakToPeak = 0;
  unsigned int signalMax = 0;
  unsigned int signalMin = 1023;
  unsigned int c, y;

  while (millis() - startMillis < SAMPLE_WINDOW){
    sample = analogRead(MIC_PIN);
    if (sample < 1024){
      if (sample > signalMax){
        signalMax = sample;
      } else if (sample < signalMin){
        signalMin = sample;
      }
    }
  }
  peakToPeak = signalMax - signalMin;

  for (int i = 0; i <= strip.numPixels() - 1; i++){
    strip.setPixelColor(i, Wheel(map(i, 0, strip.numPixels() - 1, 30, 150)));
  }
  
  c = fscale(INPUT_FLOOR, INPUT_CEILING, strip.numPixels(), 0, peakToPeak, 2);

  if (c < peak){
    peak = c;
    dotHangCount = 0;
  }

  if (c <= strip.numPixels()) 
    drawLine(strip.numPixels(), strip.numPixels() - c, strip.Color(0, 0, 0));

  y = strip.numPixels() - peak;
  strip.setPixelColor(y - 1, Wheel(map(y, 0, strip.numPixels() - 1, 30, 150)));
  strip.show();

  if (dotHangCount > PEAK_HANG){
    if (++dotCount >= PEAK_FALL){
      peak++;
      dotCount = 0;
    }
  } else {
    dotHangCount++;
  }
}
void drawLine(uint8_t from, uint8_t to, uint32_t c) {
  uint8_t fromTemp;
  if (from > to){
    fromTemp = from;
    from = to;
    to = fromTemp;
  }
  for (int i = from; i <= to; i++){
    strip.setPixelColor(i, c);
  }
}
float fscale(float originalMin, float originalMax, float newBegin,
              float newEnd, float inputValue, float curve){
  float OriginalRange = 0;
  float NewRange = 0;
  float zeroRefCurVal = 0;
  float normalizedCurVal = 0;
  float rangedValue = 0;
  boolean invFlag = 0;

  if (curve > 10) curve = 10;
  if (curve < -10) curve = -10;
  curve = (curve * -.1);
  curve = pow(10, curve);

  if (inputValue < originalMin) inputValue = originalMin;
  if (inputValue > originalMax) inputValue = originalMax;
  OriginalRange = originalMax - originalMin;

  if (newEnd > newBegin){
    NewRange = newEnd - newBegin;
  } else {
    NewRange = newBegin - newEnd;
    invFlag = 1;
  }
  
  zeroRefCurVal = inputValue - originalMin;
  normalizedCurVal = zeroRefCurVal / OriginalRange;

  if (originalMin > originalMax) return 0;
  if (invFlag == 0){
    rangedValue = (pow(normalizedCurVal, curve) * NewRange) + newBegin;
  } else {
    rangedValue = newBegin - (pow(normalizedCurVal, curve) * NewRange);
  }
  return rangedValue;
}
uint32_t Wheel(byte WheelPos){
  if (WheelPos < 85){
    return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  } else if (WheelPos < 170){
    WheelPos -= 85;
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } else {
    WheelPos -= 170;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
}
