#include <Adafruit_NeoPixel.h> //NeoPixel LEDを光らせるためのライブラリ
#define PIN 13 //LEDのDINを接続したArduinoのピン
#define NUM_LEDS 14 //LEDの数
#define BUTTON_PIN 2 //スイッチを接続したArduinoのピン

//NeoPixelライブラリの初期化
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_LEDS, PIN, NEO_GRB + NEO_KHZ800);
int buttonPushCounter = 0; //スイッチを押した回数を数えるカウンター
int buttonState = 1; //スイッチの状態
int lastButtonState = 1; //一つ前のスイッチの状態

void setup(){ //Arduino初期設定：電源ON時・リセット時に実行される
  strip.begin();
  strip.show(); //LEDを「オフ」に設定
  pinMode(BUTTON_PIN, INPUT_PULLUP); //スイッチを接続したArduinoピンの初期設定
}

void loop(){ //実行したいプログラムを記述：繰返し実行される
  buttonState = digitalRead(BUTTON_PIN);
  if (buttonState != lastButtonState){
    if (buttonState == HIGH) buttonPushCounter++;
  }
  switch (buttonPushCounter){ //カウンターの値によって光り方と色を変える
    case 1: //紫
      Sparkle(0xa1, 0x00, 0xff, 0);
      break;
    case 2: //黄色
      Strobe(0xFF, 0xFF, 0x00, 10, 15, 100);
      break;
    case 3: //ピンク
      meteorRain(0xff, 0x00, 0xff, 10, 64, true, 15);
      break;
    case 4: //ライム
      RunningLights(0x00, 0xFF, 0xBF, 10);
      break;
    case 5: //虹色
      TwinkleRandom(20, 100, false);
      buttonPushCounter = 0; //カウンターをリセット
      break;
  }
  lastButtonState = buttonState;
  delay(10);
}

//1番目の光り方
void Sparkle(byte red, byte green, byte blue, int SpeedDelay){
  int Pixel = random(NUM_LEDS);
  setPixel(Pixel, red, green, blue);
  strip.show();
  delay(SpeedDelay);
  setPixel(Pixel, 0, 0, 0);
}
//2番目の光り方
void Strobe(byte red, byte green, byte blue, int StrobeCount, int FlashDelay, int EndPause){
  for (int j = 0; j < StrobeCount; j++){
    setAll(red, green, blue);
    strip.show();
    delay(FlashDelay);
    setAll(0, 0, 0);
    strip.show();
    delay(FlashDelay);
  }
  delay(EndPause);
}
//3番目の光り方
void meteorRain(byte red, byte green, byte blue, byte meteorSize,
                byte meteorTrailDecay, boolean meteorRandomDecay, int SpeedDelay){
  setAll(0, 0, 0);
  for (int i = 0; i < NUM_LEDS + NUM_LEDS; i++){
    for (int j = 0; j < NUM_LEDS; j++){
      if ( (!meteorRandomDecay) || (random(10) > 5) ) fadeToBlack(j, meteorTrailDecay);
    }
    for (int j = 0; j < meteorSize; j++){
      if ( ( i - j < NUM_LEDS) && (i - j >= 0) ) setPixel(i - j, red, green, blue);
    }
    strip.show();
    delay(SpeedDelay);
  }
}
void fadeToBlack(int ledNo, byte fadeValue){
  uint32_t oldColor;
  uint8_t r, g, b;
  int value;

  oldColor = strip.getPixelColor(ledNo);
  r = (oldColor & 0x00ff0000UL) >> 16;
  g = (oldColor & 0x0000ff00UL) >> 8;
  b = (oldColor & 0x000000ffUL);

  r = (r <= 10) ? 0 : (int) r - (r * fadeValue / 256);
  g = (g <= 10) ? 0 : (int) g - (g * fadeValue / 256);
  b = (b <= 10) ? 0 : (int) b - (b * fadeValue / 256);

  strip.setPixelColor(ledNo, r, g, b);
}
//4番目の光り方
void RunningLights(byte red, byte green, byte blue, int WaveDelay){
  int Position = 0;
  for (int i = 0; i < NUM_LEDS * 2; i++){
    Position++;
    for (int i = 0; i < NUM_LEDS; i++){
      setPixel(i, ((sin(i + Position) * 127 + 128) / 255) * red,
                  ((sin(i + Position) * 127 + 128) / 255) * green,
                  ((sin(i + Position) * 127 + 128) / 255) * blue);
    }
    strip.show();
    delay(WaveDelay);
  }
}
//5番目の光り方
void TwinkleRandom(int Count, int SpeedDelay, boolean OnlyOne){
  setAll(0, 0, 0);
  for (int i = 0; i < Count; i++){
    setPixel(random(NUM_LEDS), random(0, 255), random(0, 255), random(0, 255));
    strip.show();
    delay(SpeedDelay);
    if (OnlyOne) setAll(0, 0, 0);
  }
  delay(SpeedDelay);
}
//共通関数
void setPixel(int Pixel, byte red, byte green, byte blue){
  strip.setPixelColor(Pixel, strip.Color(red, green, blue));
}
void setAll(byte red, byte green, byte blue){
  for (int i = 0; i < NUM_LEDS; i++){
    setPixel(i, red, green, blue);
  }
  strip.show();
}
